-------------------------------------------------------------------------------
--                 L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local daysInYear = 
{
  {0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335},
  {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334}
};

local holidayFlag =
{
  ["brewfest"             ] = NigglesHolidayBrewfest,
  ["childrensweek"        ] = NigglesHolidayChildrensWeek,
  ["darkmoonfaireelwynn"  ] = NigglesHolidayDarkmoonFaire,
  ["darkmoonfairemulgore" ] = NigglesHolidayDarkmoonFaire,
  ["darkmoonfaireterokkar"] = NigglesHolidayDarkmoonFaire,
  ["dayofthedead"         ] = NigglesHolidayDayOfTheDead,
  ["hallowsend"           ] = NigglesHolidayHallowsEnd,
  ["harvestfestival"      ] = NigglesHolidayHarvestFestival,
  ["loveintheair"         ] = NigglesHolidayLoveInTheAir,
  ["lunarfestival"        ] = NigglesHolidayLunarFestival,
  ["midsummer"            ] = NigglesHolidayMidsummer,
  ["noblegarden"          ] = NigglesHolidayNobleGarden,
  ["piratesday"           ] = NigglesHolidayPiratesDay,
  ["winterveil"           ] = NigglesHolidayWinterVeil
};

local holidayList = {};

-------------------------------------------------------------------------------
--                 L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to convert a date & time into a crude minute offset for determining
-- if a holiday is active
--
local function getHolidayTime(year, month, day, hour, min)
  if (year%4 == 0) then
    return (((((year*365)+daysInYear[1][month]+day)*24)+hour)*60)+min;
  else
    return (((((year*365)+daysInYear[2][month]+day)*24)+hour)*60)+min;
  end
end

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to get a table of the active holidays
--
function NigglesHolidaysGetActive()
  -- Local Variables
  local currentDay;
  local currentMonth;
  local currentYear;
  local currentHour;
  local currentMin;
  local currentTime;
  local activeHolidays = 0;

  -- Get the current date & time
  _, currentMonth, currentDay, currentYear = CalendarGetDate();
  currentHour, currentMin = GetGameTime();
  currentTime = getHolidayTime(currentYear, currentMonth, currentDay,
    currentHour, currentMin);

  -- Search for any active holidays
  for loop = 1, #holidayList do
    if ((currentTime >= holidayList[loop].start) and 
        (currentTime <= holidayList[loop].finish)) then
       activeHolidays = bit.bor(activeHolidays, holidayList[loop].event);
    end
  end

  return activeHolidays;
end

--
-- Function to initialise an array with holiday info for the previous, current
-- and next month
--
function NigglesHolidaysInitialise()
  -- Local Variables
  local numEvents;
  local eventCalType;
  local eventDate = {};
  local eventTexture;
  local eventFlag;
  local eventType;
  local numDays;
  local eventSeq;
  local idx;

  -- Clear any existing holiday info
  wipe(holidayList);

  -- Get the current date
  _, eventDate.month, eventDate.day, eventDate.year = CalendarGetDate();

  -- Set the current month to properly initialise the Calendar functions
  CalendarSetAbsMonth(eventDate.month, eventDate.year);

  -- Create a list of holidays in the previous, current and next months
  for monthIdx = -1, 1 do
    -- Work out the current month
    currentMonth = (eventDate.year*12)+eventDate.month+monthIdx-1;
    currentYear  = math.floor(currentMonth/12);
    currentMonth = (currentMonth%12)+1;

    -- Set the current month
    CalendarSetAbsMonth(currentMonth, currentYear);
    _, _, numDays, _ = CalendarGetMonth(0);

    -- Check each day of the month for events
    for dayIdx = 1, numDays do
      numEvents = CalendarGetNumDayEvents(0, dayIdx);
      for eventIdx = 1, numEvents do
        _, eventDate.hour, eventDate.min, eventCalType, eventSeq, eventType,
        eventTexture,  _, _ = CalendarGetDayEvent(0, dayIdx, eventIdx);
        if ((eventCalType == "HOLIDAY") and (eventType == 0)) then
          eventTexture = eventTexture:lower();
          eventFlag    = holidayFlag[eventTexture:gsub("^calendar_", "")];
          if (eventFlag ~= nil) then
            if (eventSeq == "START") then
              -- Add the start event to the holiday info
              table.insert(holidayList,
                {
                  start  = getHolidayTime(currentYear, currentMonth,
                    dayIdx, eventDate.hour, eventDate.min);
                  finish = 0xFFFFFFFF;
                  event  = eventFlag;
                });
            elseif (eventSeq == "END") then
              -- Search for a corresponding start event
              idx = 0;
              for loop = 1, #holidayList do
                if (eventFlag == holidayList[loop].event) then
                  idx = loop;
                  break;
                end
              end

              -- Add the end event to the holiday info
              if (idx == 0) then
                table.insert(holidayList,
                  {
                    start = 0x00000000;
                    event = holidayFlag[eventFlag];
                  });
                idx = #holidayList;
              end
              holidayList[idx].finish = getHolidayTime(currentYear,
                currentMonth, dayIdx, eventDate.hour, eventDate.min);
            end
          end
        end
      end
    end
  end

  return;
end
